@@ -12,8 +12,8 @@ Tools and Utilities for debugging and helping with development of Editra.
 
 """
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: dev_tool.py 66868 2011-02-09 16:01:49Z CJP $"
-__revision__ = "$Revision: 66868 $"
+__svnid__ = "$Id: dev_tool.py 72223 2012-07-28 17:35:57Z CJP $"
+__revision__ = "$Revision: 72223 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -87,29 +87,31 @@ def DEBUGP(statement):
 
     # Only print to stdout when DEBUG is active
     # Cant print to stdio if using pythonw
+    msg_type = msg.Type
     if ed_glob.DEBUG:
-        logfile = EdLogFile()
         mstr = unicode(msg)
         mstr = mstr.encode('utf-8', 'replace')
         if not PYTHONW:
             print(mstr)
+
         # Write to log file
+        logfile = EdLogFile()
         logfile.WriteMessage(mstr)
 
         # Check for trapped exceptions to print
-        if ed_glob.VDEBUG and msg.Type in ('err', 'error'):
+        if ed_glob.VDEBUG and msg_type in ('err', 'error'):
             traceback.print_exc()
             logfile.WriteMessage(traceback.format_exc())
 
     # Dispatch message to all observers
-    if msg.Type in ('err', 'error'):
+    if msg_type in ('err', 'error'):
         mtype = ed_msg.EDMSG_LOG_ERROR
         if ed_glob.VDEBUG:
             msg = LogMsg(msg.Value + os.linesep + traceback.format_exc(),
                          msg.Origin, msg.Type)
-    elif msg.Type in ('warn', 'warning'):
+    elif msg_type in ('warn', 'warning'):
         mtype = ed_msg.EDMSG_LOG_WARN
-    elif msg.Type in ('evt', 'event'):
+    elif msg_type in ('evt', 'event'):
         mtype = ed_msg.EDMSG_LOG_EVENT
     elif msg.Type in ('info', 'information'):
         mtype = ed_msg.EDMSG_LOG_INFO
@@ -127,17 +129,23 @@ class LogMsg(object):
     being expired.
 
     """
-    def __init__(self, msg, msrc='unknown', level="info"):
+    def __init__(self, msg, msrc=u"unknown", level=u"info"):
         """Create a LogMsg object
         @param msg: the log message string
         @keyword msrc: Source of message
         @keyword level: Priority of the message
 
         """
+        assert isinstance(msg, basestring)
+        assert isinstance(msrc, basestring)
+        assert isinstance(level, basestring)
         super(LogMsg, self).__init__()
 
         # Attributes
-        self._msg = dict(mstr=msg, msrc=msrc, lvl=level, tstamp=time.time())
+        self._msg = dict(mstr=DecodeString(msg),
+                         msrc=DecodeString(msrc),
+                         lvl=DecodeString(level),
+                         tstamp=time.time())
         self._ok = True
 
     def __eq__(self, other):
@@ -166,18 +174,32 @@ class LogMsg(object):
 
     def __str__(self):
         """Returns a nice formatted string version of the message"""
-        statement = DecodeString(self._msg['mstr'])
-        s_lst = [u"[%s][%s][%s]%s" % (self.ClockTime, self._msg['msrc'],
-                                      self._msg['lvl'], msg.rstrip()) 
-                 for msg in statement.split(u"\n")
+        s_lst = [u"[%s][%s][%s]%s" % (self.ClockTime, self.Origin,
+                                      self.Type, msg.rstrip()) 
+                 for msg in self.Value.split(u"\n")
                  if len(msg.strip())]
-        out = os.linesep.join(s_lst)
+        try:
+            sys_enc = sys.getfilesystemencoding()
+            out = os.linesep.join([val.encode(sys_enc, 'replace') 
+                                   for val in s_lst])
+        except UnicodeEncodeError:
+            out = repr(self)
 
         # Mark Message as have being fetched (expired)
         self._ok = False
 
         return out
 
+    def __unicode__(self):
+        """Convert to unicode"""
+        rval = u""
+        try:
+            sval = str(self)
+            rval = sval.decode(sys.getfilesystemencoding(), 'replace')
+        except UnicodeDecodeError, msg:
+            pass
+        return rval
+
     @property
     def ClockTime(self):
         """Formatted timestring of the messages timestamp"""
@@ -244,18 +266,19 @@ def DecodeString(string, encoding=None):
             rtxt = string
         return rtxt
     else:
-        # The string is already unicode so just return it
+        # The string is already Unicode so just return it
         return string
 
 #-----------------------------------------------------------------------------#
 
 class EdErrorDialog(eclib.ErrorDialog):
+    """Error reporter dialog"""
     def __init__(self, msg):
         super(EdErrorDialog, self).__init__(None, title="Error Report",
                                             message=msg)
 
         # Setup
-        self.SetDescriptionLabel(_("Error: Something unexpected happend\n"
+        self.SetDescriptionLabel(_("Error: Something unexpected hapend\n"
                                    "Help improve Editra by clicking on "
                                    "Report Error\nto send the Error "
                                    "Traceback shown below."))
