@@ -13,8 +13,8 @@ Editra Business Model Library: DirectoryMonitor
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: _dirmon.py 70229 2012-01-01 01:27:10Z CJP $"
-__revision__ = "$Revision: 70229 $"
+__cvsid__ = "$Id: _dirmon.py 71949 2012-07-03 13:41:15Z CJP $"
+__revision__ = "$Revision: 71949 $"
 
 __all__ = ['DirectoryMonitor',]
 
@@ -32,11 +32,12 @@ import fileutil
 
 class DirectoryMonitor(object):
     """Object to manage monitoring file system changes"""
-    def __init__(self):
+    def __init__(self, checkFreq=1000.0):
+        """@keyword checkFreq: check frequency in milliseconds"""
         super(DirectoryMonitor, self).__init__()
 
         # Attributes
-        self._watcher = WatcherThread(self._ThreadNotifier)
+        self._watcher = WatcherThread(self._ThreadNotifier, checkFreq=checkFreq)
         self._callbacks = list()
         self._cbackLock = threading.Lock()
         self._running = False
@@ -61,15 +62,18 @@ class DirectoryMonitor(object):
 
     # Is the monitor currently watching any directories
     Monitoring = property(lambda self: self._running)
+    Frequency = property(lambda self: self._watcher.GetFrequency(),
+                         lambda self, freq: self._watcher.SetFrequency(freq))
 
     #---- End Properties ----#
 
     def AddDirectory(self, dname):
         """Add a directory to the monitor
         @param dname: directory path
+        @return: bool - True if added, False if failed to add
 
         """
-        self._watcher.AddWatchDirectory(dname)
+        return self._watcher.AddWatchDirectory(dname)
 
     def SubscribeCallback(self, callback):
         """Subscribe a callback method to be called when changes are
@@ -102,17 +106,39 @@ class DirectoryMonitor(object):
         self._running = True
         self._watcher.start()
 
+    def Suspend(self, pause=True):
+        """Suspend background processing
+        @keyword pause: True (suspend) False (resume)
+
+        """
+        if pause:
+            self._watcher.Suspend()
+        else:
+            self._watcher.Continue()
+
+    def Refresh(self):
+        """Force a recheck of the monitored directories. This method
+        is useful for doing manual control of the refresh cycle. It is
+        ignored and does nothing when WatcherThread is set up for automatic
+        refresh cycles.
+
+        """
+        self._watcher.Refresh()
+
 #-----------------------------------------------------------------------------#
     
 class WatcherThread(threading.Thread):
     """Background thread to monitor a directory"""
-    def __init__(self, notifier):
+    def __init__(self, notifier, checkFreq=1000.0):
         """Create the WatcherThread. Provide a callback notifier method
         that will be called when changes are detected in the directory.
         The notifier will be called in the context of this thread. Notifier
         will be called with three lists of ebmlib.File objects to indicate
         the changes that have occurred.
         @param notifier: callable([added,], [deleted,], [modified,])
+        @keyword checkFreq: check frequency in milliseconds. If value is set
+                            to zero or less update checks must be manually
+                            controlled via the Refresh interface.
 
         """
         super(WatcherThread, self).__init__()
@@ -122,21 +148,27 @@ class WatcherThread(threading.Thread):
         self._notifier = notifier
         self._dirs = list() # Directories being monitored
 
-        self._freq = 1000.0 # Monitoring frequency in milliseconds
+        self._freq = checkFreq # Monitoring frequency in milliseconds
         self._continue = True
+        self._changePending = False
         self._lock = threading.Lock()
         self._suspend = False
         self._suspendcond = threading.Condition()
+        self._listEmptyCond = threading.Condition()
+        self._refreshCond = threading.Condition()
 
     def run(self):
         """Run the watcher"""
-        # TODO: wait on condition/event when dir list is empty
-        #       instead of looping per frequency interval.
         while self._continue:
             deleted = list()
             added = list()
             modified = list()
 
+            # Watch is empty so wait on things to monitor before continuing
+            if not self._dirs:
+                with self._listEmptyCond:
+                    self._listEmptyCond.wait()
+
             # Suspend processing if requested
             if self._suspend:
                 with self._suspendcond:
@@ -146,6 +178,8 @@ class WatcherThread(threading.Thread):
                 for dobj in self._dirs:
                     if not self._continue:
                         return
+                    elif self._changePending:
+                        break
 
                     # Check if a watched directory has been deleted
                     if not os.path.exists(dobj.Path):
@@ -157,24 +191,31 @@ class WatcherThread(threading.Thread):
                                                            False, True)
 
                     # Check for deletions
-                    for tobj in dobj.Files:
+                    dobjFiles = dobj.Files # optimization
+                    dobjIndex = dobjFiles.index # optimization
+                    snapFiles = snapshot.Files
+                    for tobj in dobjFiles:
                         if not self._continue:
                             return
-                        if tobj not in snapshot.Files:
+                        elif self._changePending:
+                            break
+                        if tobj not in snapFiles:
                             deleted.append(tobj)
-                            dobj.Files.remove(tobj)
+                            dobjFiles.remove(tobj)
 
                     # Check for additions and modifications
-                    for tobj in snapshot.Files:
+                    for tobj in snapFiles:
                         if not self._continue:
                             return
-                        if tobj not in dobj.Files:
+                        elif self._changePending:
+                            break
+                        if tobj not in dobjFiles:
                             # new object was added
                             added.append(tobj)
-                            dobj.Files.append(tobj)
+                            dobjFiles.append(tobj)
                         else:
-                            idx = dobj.Files.index(tobj)
-                            existing = dobj.Files[idx]
+                            idx = dobjIndex(tobj)
+                            existing = dobjFiles[idx]
                             # object was modified
                             if existing.ModTime < tobj.ModTime:
                                 modified.append(tobj)
@@ -185,22 +226,38 @@ class WatcherThread(threading.Thread):
                 self._notifier(added, deleted, modified)
 
             # Wait till next check
-            time.sleep(self._freq / 1000.0)
+            if self._freq > 0:
+                # Automatic updates
+                time.sleep(self._freq / 1000.0)
+            else:
+                # Manually controlled updates
+                with self._refreshCond:
+                    self._refreshCond.wait()
 
     #---- Implementation ----#
 
     def AddWatchDirectory(self, dpath):
         """Add a directory to the watch list
         @param dpath: directory path (unicode)
+        @return: bool - True means watch was added, False means unable to list directory
 
         """
         assert os.path.isdir(dpath)
         dobj = fileutil.Directory(dpath)
+        self._changePending = True
         with self._lock:
-            if dobj not in self._dirs:
+            if dobj not in self._dirs and os.access(dobj.Path, os.R_OK):
                 # Get current snapshot of the directory
-                dobj = fileutil.GetDirectoryObject(dpath, False, True)
+                try:
+                    dobj = fileutil.GetDirectoryObject(dpath, False, True)
+                except OSError:
+                    self._changePending = False
+                    return False
                 self._dirs.append(dobj)
+                with self._listEmptyCond:
+                    self._listEmptyCond.notify()
+        self._changePending = False
+        return True
 
     def RemoveWatchDirectory(self, dpath):
         """Remove a directory from the watch
@@ -208,9 +265,25 @@ class WatcherThread(threading.Thread):
 
         """
         dobj = fileutil.Directory(dpath)
+        self._changePending = True
         with self._lock:
             if dobj in self._dirs:
                 self._dirs.remove(dobj)
+            # Also remove any subpaths of dpath
+            toremove = list()
+            for d in self._dirs:
+                if fileutil.IsSubPath(d.Path, dpath):
+                    toremove.append(d)
+            for todel in toremove:
+                self._dirs.remove(todel)
+        self._changePending = False
+
+    def GetFrequency(self):
+        """Get the update frequency
+        @return: int (milliseconds)
+
+        """
+        return self._freq
 
     def SetFrequency(self, milli):
         """Set the update frequency
@@ -219,6 +292,14 @@ class WatcherThread(threading.Thread):
         """
         self._freq = float(milli)
 
+    def Refresh(self):
+        """Recheck the monitored directories
+        only useful when manually controlling refresh cycle of the monitor.
+
+        """
+        with self._refreshCond:
+            self._refreshCond.notify()
+
     def Shutdown(self):
         """Shut the thread down"""
         self._continue = False
