@@ -14,8 +14,8 @@ Text editor buffer view control for the main notebook
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_editv.py 69791 2011-11-19 22:02:59Z CJP $"
-__revision__ = "$Revision: 69791 $"
+__svnid__ = "$Id: ed_editv.py 71286 2012-04-26 20:59:23Z CJP $"
+__revision__ = "$Revision: 71286 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -152,13 +152,10 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         cfocus = self.FindFocus()
         if not self._focused and cfocus is self:
             # Focus has just returned to the window
-            self.SetCaretWidth(self._caret_w)
+            self.RestoreCaret()
             self._focused = True
         elif self._focused and cfocus is not self:
-            cwidth = self.GetCaretWidth()
-            if cwidth > 0:
-                self._caret_w = cwidth
-            self.SetCaretWidth(0) # Hide the caret when not active
+            self.HideCaret() # Hide the caret when not active
             self._focused = False
             self.CallTipCancel()
 
@@ -202,9 +199,21 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
     @modalcheck
     def DoReloadFile(self):
         """Reload the current file"""
-        ret, rmsg = self.ReloadFile()
+        cfile = self.GetFileName()
+        ret = True
+        rmsg = u""
+        try:
+            ret, rmsg = self.ReloadFile()
+        except Exception, msg:
+            # Unexpected error
+            wx.MessageBox(_("Failed to reload file\n\nError:\n%s") % msg,
+                          _("File read error"), wx.ICON_ERROR|wx.OK|wx.CENTER)
+            # Set modtime to prevent re-prompting of dialog regardless of error cases
+            self.SetModTime(GetFileModTime(cfile))
+            return
+
+        # Check for expected errors
         if not ret:
-            cfile = self.GetFileName()
             errmap = dict(filename=cfile, errmsg=rmsg)
             mdlg = wx.MessageDialog(self,
                                     _("Failed to reload %(filename)s:\n"
@@ -214,6 +223,9 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
             mdlg.ShowModal()
             mdlg.Destroy()
 
+        # Set modtime to prevent re-prompting of dialog regardless of error cases
+        self.SetModTime(GetFileModTime(cfile))
+
     def DoTabClosing(self):
         """Save the current position in the buffer to reset on next load"""
         if len(self.GetFileName()) > 1:
@@ -388,14 +400,6 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
 
     #---- End EdTab Methods ----#
 
-    def IsNonCode(self, pos):
-        """Is the passed in position in a non code region
-        @param pos: buffer position
-        @return: bool
-
-        """
-        return self.IsComment(pos) or self.IsString(pos)
-
     def OnConfigMsg(self, msg):
         """Update config based on profile changes"""
         mtype = msg.GetType()[-1]
@@ -409,6 +413,10 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         elif mtype == 'AUTO_COMP_EX':
             self.ConfigureAutoComp()
             return
+        elif mtype == 'CARETWIDTH':
+            if self.GetCaretWidth(): # check that it is not hidden
+                self.RestoreCaret()
+            return
         elif mtype in ('VI_EMU', 'VI_NORMAL_DEFAULT'):
             self.SetViEmulationMode(Profile_Get('VI_EMU'),
                                     Profile_Get('VI_NORMAL_DEFAULT'))
@@ -435,7 +443,8 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
                    'SHOW_EOL'   : self.SetViewEOL,
                    'SHOW_LN'    : self.ToggleLineNumbers,
                    'SHOW_WS'    : self.SetViewWhiteSpace,
-                   'WRAP'       : self.SetWrapMode                   }
+                   'WRAP'       : self.SetWrapMode }
+
         if mtype in cfgmap:
             cfgmap[mtype](Profile_Get(mtype))
             return
@@ -577,6 +586,7 @@ class EdEditorView(ed_stc.EditraStc, ed_tab.EdTabBase):
         if result == wx.ID_YES:
             self.DoReloadFile()
         else:
+            # Set modtime to prevent re-prompting of dialog
             self.SetModTime(GetFileModTime(cfile))
 
     def SetLexer(self, lexer):
